//*************************************************************************
//* --------------
//* CDCClusterHist
//* --------------
//* (Description)
//* 	Clasess to create FADC cluster related histograms.
//* (Update Record)
//*	1999/12/06  K.Fujii	Original version.
//*************************************************************************


#include "JSFSteer.h"
#include "CDCClusterHist.h"
#include <iostream.h>

ClassImp(CDCClusterHist)

const Int_t CDCClusterHist::kZoneX   = 4;
const Int_t CDCClusterHist::kZoneY   = 3;
const Int_t CDCClusterHist::kNh = kZoneX*kZoneY;

//___________________________________________________________________________
CDCClusterHist::CDCClusterHist(const Char_t *name, const Char_t *title)
	: JSFModule(name, title)
{
   hdef2 = new THdef2[kNh];
   Char_t st[256], std[256];
   Int_t i;
   for (i = 0; i < kNh; i++) {
      sprintf(st,"CDCClusterHist.hPh%2.2d",i);
      sprintf(std,"w-sum for ch %2.2d, 200,0.,1000.,120,0.,12000",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdef2[i].title,
			&hdef2[i].nxbin, &hdef2[i].xmin, &hdef2[i].xmax,
			&hdef2[i].nybin, &hdef2[i].ymin, &hdef2[i].ymax);
   }
   hPh = new TH2F* [kNh];
}

//___________________________________________________________________________
CDCClusterHist::~CDCClusterHist()
{
   delete [] hdef2;
   Int_t i;
   for (i = 0; i < kNh; i++) delete hPh[i];
   delete [] hPh; 
}

//___________________________________________________________________________
Bool_t CDCClusterHist::Initialize()
{
   TDirectory *last=gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < kNh; i++) {
      hPh[i] = new TH2F(hdef2[i].title,hdef2[i].title,
      			hdef2[i].nxbin,hdef2[i].xmin,hdef2[i].xmax,
      			hdef2[i].nybin,hdef2[i].ymin,hdef2[i].ymax);
      cout << i << " "       << hdef2[i].title 
      		<< " nx "   << hdef2[i].nxbin
      		<< " xmin " << hdef2[i].xmin
      		<< " xmax " << hdef2[i].xmax
      		<< " ny "   << hdef2[i].nybin
      		<< " ymin " << hdef2[i].ymin
      		<< " ymax " << hdef2[i].ymax
      		<< endl;
   }
   hc1=new TCanvas("hc1","Canvas 1",20, 20, 200*kZoneX, 200*kZoneY);
   hc1->Divide(kZoneX,kZoneY);
   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t CDCClusterHist::Process(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   CDCClusterMaker *cm  = (CDCClusterMaker*)gJSF->FindModule("CDCClusterMaker");
   CDCClusterBuf   *clb = (CDCClusterBuf *)cm->EventBuf();
   TObjArray *clsp = clb->GetClusters();
   TIter next(clsp);
   CDCCluster *clp;
   while ((clp = (CDCCluster *)next())) {
#if 0
      Int_t    n    = clp->GetStationNo();
      Double_t tim  = clp->GetStartTime();
#endif
      Int_t    a    = clp->GetChannelNo();
      // Double_t adc  = clp->GetPeak();
      Double_t adc  = clp->GetSum();
      Double_t wid  = clp->GetWidth();
#if 0
      cerr << n << " " << a << " " << adc << " " << tim << endl;
#endif
      if ( a >= 0 && a < CDCClusterHist::kNh ) {
      	 hPh[a]->Fill(wid,adc,1.0);
      }
   }
   if (!(gROOT->IsBatch())) Draw();
   last->cd();
   return kTRUE;
}

//___________________________________________________________________________
void CDCClusterHist::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < CDCClusterHist::kNh; i++) {
      hc1->cd(i+1);
      hPh[i]->Draw();
      hc1->Update();
   }
   last->cd();
}

