#include "UserAnalysis.h"

ClassImp(UserAnalysis)

//______________________________________________________________________________
// ---------------
//  Ctor and Dtor
// ---------------
//
UserAnalysis::UserAnalysis(BabyDisplay *display = 0,
		           CDCFADCCalibration *calib = 0) :
	fDisplay(display), fCalib(calib) {}

UserAnalysis::~UserAnalysis() {}

//______________________________________________________________________________
// --------------
//  Baby Chamber
// --------------
//

void UserAnalysis::InitRawFADC()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  for (Int_t i = 0; i < kNmods; i++) {
     sprintf(st,"hFADC%2.2d",i);
#if defined(__COSM2000__)
     if (i < 8) hFADC[i] = new TH1F(st,"",2732,0.,8196.);
     else	hFADC[i] = new TH1F(st,"",4098,500.,8696.);
#elif defined(__HIROSHIMA__)
     if (i < 8) hFADC[i] = new TH1F(st,"",2200,0.,6600.);
     else	hFADC[i] = new TH1F(st,"",3300,0.,6600.);
#else
     if (i < 8) hFADC[i] = new TH1F(st,"",2200,0.,6600.);
     else	hFADC[i] = new TH1F(st,"",3300,0.,6600.);
#endif
#if !defined(__COSM2000__) && !defined(__HIROSHIMA__)
     sprintf(st,"hTPK%2.2d",i);
     hTPK[i] = new TH2F(st,"",200,0.,6600.,255,0.,255.);
     sprintf(st,"hTQ%2.2d",i);
     hTQ[i] = new TH2F(st,"",200,0.,6600.,200,0.,10000.);
     sprintf(st,"hTOFTQ%2.2d",i);
     hTOFTQ[i] = new TH2F(st,"",200,0.,6600.,200,0.,10000.);
#endif
  }
  last->cd();
}

void UserAnalysis::DrawRawFADC()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");

  CDCTrackMaker  *tm = (CDCTrackMaker*)gJSF->FindModule("CDCTrackMaker");
  CDCTrackBuf    *tb = (CDCTrackBuf *)tm->EventBuf();
  TObjArray      *ta = tb->GetTracks();

  CDCTrack *tp;
  TIter nexttrack(ta);
  
//#if !defined(__COSM2000__) && !defined(__HIROSHIMA__)
  while ((tp = (CDCTrack *)nexttrack())) {
    CDCTrackHit *trkhitp;
    CDCHit *hitp;
    TIter nexttrkhit(tp);
    while ((trkhitp = (CDCTrackHit *)nexttrkhit())) {
      hitp = trkhitp->GetCDCHit();
      Int_t layerNo = hitp->GetLayerNo();
      Int_t cellNo  = hitp->GetCellNo();
      Int_t wireNo  = hitp->GetWireNo();
      Int_t a       = 5*layerNo + wireNo;  // works only for baby
      if (cellNo == 1) a = 10 + wireNo;    // works only for baby
      if (a < 0 || a > 9) continue;        // works only for baby
      Double_t time = hitp->GetCluster()->GetStartTime();
      if (a >= 5) time = kFullDriftTime - time;
      Double_t peak = hitp->GetCluster()->GetPeak();
      Double_t sum  = hitp->GetCluster()->GetSum();
      cerr << "a, Peak, Sum : " << a << " " << peak << " " << sum << endl;
      hTPK[a]->Fill(time,peak);
      hTQ[a]->Fill(time,sum);
    }
  }
//#endif
    
  if (!(fDisplay->IsInLoop())) {


    TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
    da->cd();
    
    Int_t page = fDisplay->GetPage();
    switch (page) {
      case 0: {
        CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
        CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
        CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
        CDCHitMaker    *hm = (CDCHitMaker*)gJSF->FindModule("CDCHitMaker");
        CDCHitBuf      *hb = (CDCHitBuf *)hm->EventBuf();
        TObjArray      *la = hb->GetHitLayers();

        Int_t             fmtid = br->GetHeader()->GetFormatID();
        Int_t              ncdc = evt->GetNADCData();
        TClonesArray   *adcdata = evt->GetADCData();

        for (Int_t i = 0; i < kNmods; i++) {
          hFADC[i]->Reset();
        }
        Int_t npres;
        if (gJSF->GetRunNumber() < 598) 	npres = 20;
        else				npres = 100;
        for (Int_t i = 0; i < ncdc; i++) {
          CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
          Int_t a;
          Int_t id;
          Int_t cr = 0;               // We have only 1 crate now.
          Int_t st = adcdatum->GetStationNo();
          Int_t ch = adcdatum->GetChannelNo();
          if (st > 13) break;              // TEMPORARY REMEDY FOR BROKEN DATA
          if (st <=13 && ch > 1) continue; // TEMPORARY REMEDY FOR BROKEN DATA
          if (fmtid == 0) a = ch;
          else            br->EChannelToDChannel(cr,st,ch,id,a);
          Double_t ped  = fCalib->GetPedestal(st,ch);
          Double_t gain = fCalib->GetGain(st,ch);      
          Double_t tped = fCalib->GetTimePedestal(st,ch) + npres;
          Double_t tskl = fCalib->GetTimeScale(st,ch);
          Double_t adc  = (ped - adcdatum->GetADC())*gain;
          Double_t time = (adcdatum->GetTime() - tped)*tskl;
          if (a >= 5) time = kFullDriftTime - time;
          if ( id == 0 && a >= 0 && a < kNmods ) {
            hFADC[a]->Fill(time,adc);
          }
        }

    	da->Divide(1,kNmods);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hFADC[i]->GetXaxis()->SetTickLength(0.1);
      	  hFADC[i]->GetYaxis()->SetTickLength(0.005);
      	  hFADC[i]->GetXaxis()->SetNdivisions(30);
      	  hFADC[i]->GetYaxis()->SetNdivisions(4); 
      	  hFADC[i]->GetXaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->GetYaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->Draw();
    	}

    	CDCHitLayer *layerp;
    	TIter nextLayer(la);
    	while ((layerp = (CDCHitLayer *)nextLayer())) {
      	  TIter nextCell(layerp);
      	  CDCHitCell *cellp;
      	  while ((cellp = (CDCHitCell *)nextCell())) {
	    TIter nextWire(cellp);
	    CDCHitWire *wirep;
	    while ((wirep = (CDCHitWire *)nextWire())) {
	      Int_t layerNo = wirep->GetLayerNo();
	      Int_t cellNo  = wirep->GetCellNo();
	      Int_t wireNo  = wirep->GetWireNo();
	      Int_t a       = 5*layerNo + wireNo;  // works only for baby
	      if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	      if (a < 0 || a > 9) continue;        // works only for baby
	      Char_t str[256];
	      sprintf(str,"hHit%2.2d",a);
	      DeleteGraph(str);
	      TGraph *gr = new TGraph();
	      gr->SetName(str);
	      da->cd(a+1);
	      CDCHit *hitp;
	      TIter nextHit(wirep);
	      Int_t nhits = 0;
	      while ((hitp = (CDCHit *)nextHit())) {
	        Double_t time = hitp->GetCluster()->GetStartTime();
	        if (a >= 5) time = kFullDriftTime - time;
	        gr->SetPoint(nhits++,time,0.);
	        gr->SetFillColor(19);
	        gr->SetMarkerColor(1);
	        gr->SetMarkerStyle(23);
	        gr->SetMarkerSize(1.5);
	        gr->Draw("P");
	      }
	    }
          }
        }

        nexttrack.Reset();
        Int_t ntrks = 0;
        Int_t ntrkhits = 0;
        while ((tp = (CDCTrack *)nexttrack())) {
          ntrks++;
          CDCTrackHit *trkhitp;
          CDCHit *hitp;
          TIter nexttrkhit(tp);
          while ((trkhitp = (CDCTrackHit *)nexttrkhit())) {
	    hitp = trkhitp->GetCDCHit();
	    Int_t layerNo = hitp->GetLayerNo();
	    Int_t cellNo  = hitp->GetCellNo();
	    Int_t wireNo  = hitp->GetWireNo();
	    Int_t a       = 5*layerNo + wireNo;  // works only for baby
	    if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	    if (a < 0 || a > 9) continue;        // works only for baby
	    Double_t time = hitp->GetCluster()->GetStartTime();
	    if (a >= 5) time = kFullDriftTime - time;
	    da->cd(a+1);
	    Char_t str[256];
	    sprintf(str,"hTrackHit%2.2d%3.3d",a,ntrkhits++);
	    DeleteGraph(str);
	    TGraph *gr = new TGraph();
	    gr->SetName(str);
	    gr->SetPoint(0,time,0.);
	    gr->SetFillColor(19);
	    gr->SetMarkerColor((1+ntrks)%50);
	    gr->SetMarkerStyle(8);
	    gr->SetMarkerSize(0.8);
	    gr->Draw("P");
          }
        }
    	break;
      }
#if !defined(__COSM2000__) && !defined(__HIROSHIMA__)
      case 1: {
    	da->Divide(kNmods/2,4);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hTPK[i]->GetXaxis()->SetTickLength(0.04);
      	  hTPK[i]->GetYaxis()->SetTickLength(0.05);
      	  hTPK[i]->GetXaxis()->SetNdivisions(5);
      	  hTPK[i]->GetYaxis()->SetNdivisions(5); 
      	  hTPK[i]->GetXaxis()->SetLabelSize(0.10); 
      	  hTPK[i]->GetYaxis()->SetLabelSize(0.10); 
      	  hTPK[i]->Draw();
      	  da->cd(i+11);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hTQ[i]->GetXaxis()->SetTickLength(0.04);
      	  hTQ[i]->GetYaxis()->SetTickLength(0.05);
      	  hTQ[i]->GetXaxis()->SetNdivisions(5);
      	  hTQ[i]->GetYaxis()->SetNdivisions(5); 
      	  hTQ[i]->GetXaxis()->SetLabelSize(0.10); 
      	  hTQ[i]->GetYaxis()->SetLabelSize(0.10); 
      	  hTQ[i]->Draw();
    	}
        break;
      }

#if 0      
      case 4: {
        CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
        CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
        CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

        Int_t             fmtid = br->GetHeader()->GetFormatID();
        Int_t              ncdc = evt->GetNADCData();
        TClonesArray   *adcdata = evt->GetADCData();
        
        Double_t tof = 0.;
        
     	
      	
      	
    	da->Divide(kNmods/2,4);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      }
#endif
#endif
    }
    da->cd();
    da->Update();
  }
  last->cd();
}

//______________________________________________________________________________
// ------------
//  Hodoscopes
// ------------
//

#if defined(__COSM2000__) || defined(__HIROSHIMA__)
void UserAnalysis::InitCounters()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  for (Int_t i = 0; i < kNhodos-1; i++) {
     sprintf(st,"hHADC%2.2d",i);
     hHADC[i] = new TH1F(st,"",128,0.,2048.);
  }
  hHod1     = new TH1F("hHod1","Hodoscope 1",16,0.,16.);
  hHod2     = new TH1F("hHod2","Hodoscope 2",16,0.,16.);
  hHod12    = new TH2F("hHod12","Hod1 v.s. Hod2",16,0.,16.,16,0.,16.); 
  hHodADC12 = new TH2F("hHodADC12","SumHod1 v.s. SumHod2",
  			128,0.,8192.,128,0.,8192.); 
  hHod12lo = new TH2F("hHodADC12lo","SumHod1 v.s. SumHod2 (single)",
  			128,0.,8192.,128,0.,8192.); 
  hHod12hi = new TH2F("hHodADC12hi","SumHod1 v.s. SumHod2 (double)",
  			128,0.,8192.,128,0.,8192.); 
  hTC1TC2   = new TH2F("hTC1TC2","TC1 v.s. TC2",
  			128,0.,2048.,128,0.,2048.); 
  hMonHod   = new TH2F("hMonHod","Mon v.s. Hod",
  			128,0.,2048.,128,0.,16384.); 
  hHod1hs   = new TH2F("hHod1hs","Hod1: Hard v.s. Soft",
  			128,0.,2048.,128,0.,16384.); 
  hNHod12   = new TH2F("hNHod12","Nhod1 v.s. Nhod2",
  			16,0.,16.,16,0.,16.); 
  hMonCsi   = new TH2F("hMonCsi","Mon v.s. Csi",
  			128,0.,2048.,128,0.,2048.);
  last->cd();
}

void UserAnalysis::DrawCounters()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
  CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
  CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t 	    fmtid = br->GetHeader()->GetFormatID();
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  hHod1->Reset();
  hHod2->Reset();

  Int_t    h1[16], h2[16];
  Double_t adc1[16], adc2[16], adc1pk[18], adc2pk[18];
  for (Int_t i = 0; i < 18; i++) adc1pk[i] = adc2pk[i] = 0;
  Int_t    n1 = 0;
  Int_t    n2 = 0;
  Double_t adcsum1 = 0;
  Double_t adcsum2 = 0;
  Double_t adcmon  = 0;
  Double_t adccsi  = 0;
  Double_t adctc1  = 0;
  Double_t adctc2  = 0;
  Int_t ic1st = fDisplay->SkipChamberData();
  for (Int_t i = ic1st; i < ncdc; i++) {
     CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = adcdatum->GetStationNo();
     Int_t ch = adcdatum->GetChannelNo();
     if (st == 15) cerr << "Station No." << st << " required !" << endl;
     if (st == 15) break;             // TEMPORARY REMEDY FOR BROKEN DATA
     if (st <=13 && ch > 1) continue; // TEMPORARY REMEDY FOR BROKEN DATA
     if (st <=13) continue;	      // skip chamber data
     if (fmtid == 0) a = ch;
     else            br->EChannelToDChannel(cr,st,ch,id,a);
     if (id == 0) continue;
     
     Double_t adc  = adcdatum->GetADC();
     if ( id != 0 && a >= 0 && a < kNhodos-1 ) {	// hodoscopes
        if (adc > kHodPed[a] + kHodThresh) {
           hHADC[a]->Fill(adc);
           if (a < kNhodos/2) {
              hHod1->Fill(a,adc);
              adc1[n1]    = adc - kHodPed[a];
	      adc1pk[a+1] = adc1[n1];
              adcsum1    += adc1[n1];
              h1[n1++]    = a;
           } else {
              hHod2->Fill(a%(kNhodos/2),adc);
              adc2[n2]              = adc - kHodPed[a];
              adc2pk[a+1-kNhodos/2] = adc2[n2];
              adcsum2              += adc2[n2];
              h2[n2++]              = a%(kNhodos/2);
           }
        }
     } else if ( id != 0 && a >= kNhodos-1 ) {	// counters
        switch (a) {
       	   case 31: {
       	  	adctc1 = adc;
       	  	break;
       	   }
       	   case 33: {
       	  	adctc2 = adc;
       	  	break;
       	   }
       	   case 34: {
       	  	adcmon = adc;
       	  	break;
       	   }
       	   case 35: {
       	  	adccsi = adc;
       	  	break;
       	   }
       	   default:
       	   	break;
        }
     }
  }

  Int_t npk1 = 0;
  Int_t npk2 = 0;
  for (Int_t i = 1; i <= kNhodos/2; i++) {
     if (adc1pk[i] > adc1pk[i-1] && adc1pk[i] >= adc1pk[i+1]) npk1++;
     if (adc2pk[i] > adc2pk[i-1] && adc2pk[i] >= adc2pk[i+1]) npk2++;
  }
#if 0
  cerr << " npk1 = " << npk1 << " npk2 = " << npk2
       << " adcsum1 = " << adcsum1 << endl;
#endif
  for (Int_t i = 0; i < n1; i++) {
     for (Int_t j = 0; j < n2; j++) {
     	hHod12->Fill(h1[i],h2[j],adc1[i]*adc2[j]);
     }
  }
  hHodADC12->Fill(adcsum1,adcsum2);
  hTC1TC2->Fill(adctc1,adctc2);
  hMonHod->Fill(adcmon,adcsum1+adcsum2);
  hHod1hs->Fill(adcmon,adcsum1);
  hNHod12->Fill(n1+0.5,n2+0.5);
  hMonCsi->Fill(adcmon,adccsi);
  if (npk1 < 2 && npk2 < 2) {
     hHod12lo->Fill(adcsum1,adcsum2);
  } else if ( npk1 > 1 && npk2 > 1) {
     hHod12hi->Fill(adcsum1,adcsum2);
  }
  
  if (!(fDisplay->IsInLoop())) {  
     TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
     da->cd();
     Int_t page = fDisplay->GetPage();
     switch (page) {
        case 1: {
           da->Divide(1,2);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHod1->Draw();
           da->cd(2);
           hHod2->Draw();
           break;
        }
        case 2: {
           da->Divide(8,4);
           for (Int_t i = 0; i < kNhodos-1; i++) {
              da->cd(i+1);
              gStyle->SetOptStat(0);
              gPad->SetTopMargin(0.01);
              gPad->SetBottomMargin(0.2);
              hHADC[i]->GetXaxis()->SetTickLength(0.04);
              hHADC[i]->GetYaxis()->SetTickLength(0.05);
              hHADC[i]->GetXaxis()->SetNdivisions(5);
              hHADC[i]->GetYaxis()->SetNdivisions(5); 
              hHADC[i]->GetXaxis()->SetLabelSize(0.10); 
              hHADC[i]->GetYaxis()->SetLabelSize(0.10); 
              hHADC[i]->Draw();
              // da->Update();
           }
           break;
        }
        case 3: {
           da->Divide(3,4);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHod12->Draw();
           da->cd(2);
           DeleteHist("hHod12_px");
           hHod12->ProjectionX()->Draw();
           da->cd(3);
           DeleteHist("hHod12_py");
           hHod12->ProjectionY()->Draw();
           da->cd(4);
           hHodADC12->Draw();
           da->cd(5);
           DeleteHist("hHodADC12_px");
           hHodADC12->ProjectionX()->Draw();
           da->cd(6);
           DeleteHist("hHodADC12_py");
           hHodADC12->ProjectionY()->Draw();
           da->cd(7);
#if 1
           DeleteHist("hTC1TC2_px");
           hTC1TC2->ProjectionX()->Draw();
#else
           hTC1TC2->Draw();
#endif
           da->cd(8);
#if 1
           hHod1hs->Draw();
#else
           hMonHod->Draw();
#endif
           da->cd(9);
           hNHod12->Draw();
           da->cd(10);
           hMonCsi->Draw();
#if 1
	   da->cd(11);
	   hHod12lo->Draw();
	   da->cd(12);
	   hHod12hi->Draw();
#endif
           break;
        }
        default:
           break;
     }
     da->cd(); da->Update();
  }
  last->cd();
}

void UserAnalysis::UpdateCounterInfo()
{
  CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
  CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
  CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
  
  Int_t 	    fmtid = br->GetHeader()->GetFormatID();
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t i;
  Int_t tc1adc = 0;
  Int_t tc2adc = 0;
  Int_t hod1sumadc = 0;
  Int_t csiadc = 0;
  for (i = 0; i < ncdc; i++) {
    CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
    Int_t st = adcdatum->GetStationNo();
    if (st != 20) continue;
    Int_t a;
    Int_t id;
    Int_t cr = 0;               // We have only 1 crate now.
    Int_t ch = adcdatum->GetChannelNo();
    if (fmtid == 0) a = ch;
    else            br->EChannelToDChannel(cr,st,ch,id,a);
    Double_t adc  = adcdatum->GetADC();

    switch (a) {
    case 31: {
      tc1adc = (Int_t)adc;
      //      cout << "ADC of TC1 = " << tc1adc << endl;
      break;
    }
    case 33: {
      tc2adc = (Int_t)adc;
      //      cout << "ADC of TC2 = " << tc2adc << endl;
      break;
    }
    case 34: {
      hod1sumadc = (Int_t)adc;
      //      cout << "ADC of HOD1 Sum = " << hod1sumadc << endl;
      break;
    }
    case 35: {
      csiadc = (Int_t)adc;
      //      cout << "ADC of CsI = " << csiadc << endl;
      break;
    }
    default:
      break;
    }
  }
  Char_t cntinfo[256];
  if (gJSF->GetRunNumber() < 858) {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
	    tc1adc, tc2adc, csiadc);
  } else {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  HOD1Sum = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
	    tc1adc, hod1sumadc, tc2adc, csiadc);
  }
  TGString *str = new TGString(cntinfo);
  fDisplay->UpdateCounterInfo(str);
}

#else
// ---------
// dE/dx Exp
// ---------
void UserAnalysis::InitCounters()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");

  hTOFADCTOF1 = new TH2F("hTOFADCTOF1","TOF v.s. ADC_TOF1",
  			128,0.,2048.,128,0.,2048.); 
  hTOFADCTOF2 = new TH2F("hTOFADCTOF2","TOF v.s. ADC_TOF2",
  			128,0.,2048.,128,0.,2048.); 
  hADCTOF12 = new TH2F("hADCTOF12","ADC_TOF1 v.s. ADC_TOF2",
  			128,0.,2048.,128,0.,2048.); 
  hTC1TC2   = new TH2F("hTC1TC2","TC1 v.s. TC2",
  			128,0.,2048.,128,0.,2048.); 
  hCH1CH2   = new TH2F("hCH1CH2","CH1 v.s. CH2",
  			128,0.,2048.,128,0.,2048.); 
  hCH1PBG   = new TH2F("hCH1PBG","CH1 v.s. PbG",
  			128,0.,2048.,128,0.,2048.); 
  last->cd();
}

void UserAnalysis::DrawCounters()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
  CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
  CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t 	    fmtid = br->GetHeader()->GetFormatID();
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Double_t tof     = 0.;
  Double_t adctof1 = 0.;
  Double_t adctof2 = 0.;
  Double_t adctc1  = 0.;
  Double_t adctc2  = 0.;
  Double_t adcch1  = 0.;
  Double_t adcch2  = 0.;
  Double_t adcpbg  = 0.;
  Int_t ic1st = fDisplay->SkipChamberData();
  for (Int_t i = ic1st; i < ncdc; i++) {
     CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = adcdatum->GetStationNo();
     Int_t ch = adcdatum->GetChannelNo();
     if (st == 15) {
     	cerr << "Station No." << st << " required !" << endl;
        break;             	      // TEMPORARY REMEDY FOR BROKEN DATA
     }
     if (st <=13 && ch > 1) continue; // TEMPORARY REMEDY FOR BROKEN DATA
     if (fmtid == 0) a = ch;
     else            br->EChannelToDChannel(cr,st,ch,id,a);
     if (id == 0) continue;
     
     Double_t adc  = adcdatum->GetADC();
                           
     if ( id == 3 ) {	// TOF
       if ( a == 0 ){
        tof = adcdatum->GetTime();
        cerr << "ID, a, TOF: "<< id << " " << a << " " << tof << endl;
       }
     } else if ( id != 0 ) {	// counters
        switch (a) {
       	   case 0: {
       	  	adctof1 = adc;
       	  	break;
       	   }
       	   case 1: {
       	  	adctof2 = adc;
       	  	break;
       	   }
       	   case 2: {
       	  	adctc1  = adc;
       	  	break;
       	   }
       	   case 3: {
       	  	adctc2  = adc;
       	  	break;
       	   }
           case 4: {
       	  	adcch1  = adc;
       	  	break;
       	   }
           case 5: {
       	  	adcch2  = adc;
       	  	break;
       	   }
           case 6: {
       	  	adcpbg  = adc;
       	  	break;
       	   }
      	   default:
       	   	break;
        }
     }
  }

  hTOFADCTOF1->Fill(tof,adctof1);
  hTOFADCTOF2->Fill(tof,adctof2);
  hADCTOF12->Fill(adctof1,adctof2);
  hTC1TC2->Fill(adctc1,adctc2);
  hCH1CH2->Fill(adcch1,adcch2);
  hCH1PBG->Fill(adcch1,adcpbg);

  if (!(fDisplay->IsInLoop())) {  
     TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
     da->cd();
     Int_t page = fDisplay->GetPage();
     switch (page) {
        case 2: {
           da->Divide(3,4);
           gStyle->SetOptStat(0);
           da->cd(1);
           hTOFADCTOF1->Draw();
           da->cd(2);
           DeleteHist("hTOFADCTOF1_px");
           hTOFADCTOF1->ProjectionX()->Draw();
           da->cd(3);
           DeleteHist("hTOFADCTOF1_py");
           hTOFADCTOF1->ProjectionY()->Draw();
           da->cd(4);
           hTOFADCTOF2->Draw();
           da->cd(5);
           DeleteHist("hTOFADCTOF2_py");
           hTOFADCTOF2->ProjectionY()->Draw();
           da->cd(6);
	   hADCTOF12->Draw();
           da->cd(7);
           hTC1TC2->Draw();
           da->cd(8);
           DeleteHist("hTC1TC2_px");
           hTC1TC2->ProjectionX()->Draw();
           da->cd(9);
           DeleteHist("hTC1TC2_py");
           hTC1TC2->ProjectionY()->Draw();
           da->cd(10);
           hCH1CH2->Draw();
           da->cd(11);
           hCH1PBG->Draw();
           da->cd(12);
           DeleteHist("hCH1PBG_py");
           hCH1PBG->ProjectionY()->Draw();
           break;
        }
        default:
           break;
     }
     da->cd(); da->Update();
  }
  last->cd();
}

void UserAnalysis::UpdateCounterInfo()
{
  CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
  CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
  CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
  
  Int_t 	    fmtid = br->GetHeader()->GetFormatID();
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t i;
  Int_t tof    = 0;
  Int_t adctc1 = 0;
  Int_t adctc2 = 0;
  Int_t adcch1 = 0;
  Int_t adcch2 = 0;
  Int_t adcpbg = 0;
  
  for (i = 0; i < ncdc; i++) {
    CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
    Int_t st = adcdatum->GetStationNo();
    if (st < 16) continue;
    Int_t a;
    Int_t id;
    Int_t cr = 0;               // We have only 1 crate now.
    Int_t ch = adcdatum->GetChannelNo();
    if (fmtid == 0) a = ch;
    else            br->EChannelToDChannel(cr,st,ch,id,a);
    Int_t adc  = (Int_t)adcdatum->GetADC();
    if (id == 3) {
      if ( a == 0 ) {	
        tof = (Int_t)adcdatum->GetTime();
        }
    } else {
      switch (a) {
        case 2: {
          adctc1 = adc;
          break;
        }
        case 3: {
          adctc2 = adc;
          break;
        }
        case 4: {
          adcch1 = adc;
          break;
        }
        case 5: {
          adcch2 = adc;
          break;
        }
        case 6: {
          adcpbg = adc;
          break;
        }
        default:
          break;
      }
    }
  }
  Char_t cntinfo[256];
  sprintf(cntinfo,
          "TOF: %4.4d  ADC: TC1 = %4.4d, TC2 = %4.4d, CH1 = %4.4d, Pb = %4.4d",
	  tof, adctc1, adctc2, adcch1, adcpbg);

  TGString *str = new TGString(cntinfo);
  fDisplay->UpdateCounterInfo(str);
}
#endif

//______________________________________________________________________________
// ----------
//  Steering
// ----------
//

void UserAnalysis::Initialize()
{
  // cerr << "User's Initialize() is called" << endl;
  InitRawFADC();
  InitCounters();
}

void UserAnalysis::DrawHist()
{
  // cerr << "DrawHist() is called" << endl;
  Int_t page = fDisplay->GetPage();
  switch (page) {
     case 3:
     case 2:
#if defined(__COSM2000__) || defined(__HIROSHIMA__)
     case 1:
#endif
     	DrawCounters();
        break;
#if !defined(__COSM2000__) && !defined(__HIROSHIMA__)
     case 1:
#endif
     case 0:
  	DrawRawFADC();
  	break;
     default:
        break;
  }
}


//______________________________________________________________________________
// ----------
//  Utility
// ----------
//

void UserAnalysis::DeleteHist(Char_t * name)
{
  TDirectory *last = gDirectory;
  TH1D *h = (TH1D *)gROOT->FindObject(name);
  if (h) delete h;
  last->cd();
}

void UserAnalysis::DeleteGraph(Char_t * name)
{
  TDirectory *last = gDirectory;
  TGraph *gr = (TGraph *)gROOT->FindObject(name);
  if (gr) delete gr;
  last->cd();
}
