#ifndef __USERANALYSIS__
#define __USERANALYSIS__

#include "BabyDisplay.h"
#include "TStyle.h"
#include "TH1.h"
#include "TH2.h"
#include "TGraph.h"
#include "TMath.h"
#include "JSFSteer.h"
#include "CDCTrackMaker.h"

const Int_t  kNmods  = 10;
const Int_t  kFullDriftTime = 6492;

#if defined(__COSM2000__) || defined(__HIROSHIMA__)
const Int_t  kNhodos = 32;
const Double_t  kHodThresh = 30.;
#if 0
const Double_t  kHodPed[] = {110., 110., 103., 107.,   102., 129., 117., 125.,
                             121., 113., 120., 118.,   100.,  99.,  97.,  95.,
                             105.,  82.,  89., 100.,   105.,  98., 104., 136.,
                             109., 109., 111., 102.,   104., 107., 111., 116.};
#else
const Double_t  kHodPed[] = {135., 137., 133., 134.,   134., 134., 142., 143.,
			     140., 137., 144., 145.,   131., 129., 133., 129.,
			     131., 111., 122., 127.,   131., 134., 132., 129.,
			     132., 137., 142., 130.,   139., 135., 139., 134.};
#endif
#endif

class UserAnalysis {
public:
	UserAnalysis(BabyDisplay *display, CDCFADCCalibration *calib);
	virtual ~UserAnalysis();

	void InitRawFADC();
	void DrawRawFADC();
	void InitCounters();
	void DrawCounters();
	void UpdateCounterInfo();

	void Initialize();
	void DrawHist();

	void DeleteHist(Char_t *);
	void DeleteGraph(Char_t *);

private:
        BabyDisplay 	   *fDisplay;	//!
        CDCFADCCalibration *fCalib;	//!
#if defined(__COSM2000__) || defined(__HIROSHIMA__)
	TH1F        *hHADC[kNhodos];	//!
	TH1F        *hHod1;		//!
	TH1F        *hHod2;		//!
	TH2F        *hHod12;		//!
	TH2F        *hHodADC12;		//!
	TH2F        *hHod12lo;		//!
	TH2F        *hHod12hi;		//!
	TH2F        *hTC1TC2;		//!
	TH2F        *hMonHod;		//!
	TH2F        *hHod1hs;		//!
	TH2F        *hNHod12;		//!
	TH2F        *hMonCsi;		//!
	TH1F        *hFADC[kNmods];	//!
#else
	TH2F        *hTOFADCTOF1;	//!
	TH2F        *hTOFADCTOF2;	//!
	TH2F        *hADCTOF12;		//!
	TH2F        *hTC1TC2;		//!
	TH2F        *hCH1CH2;		//!
	TH2F        *hCH1PBG;		//!
	TH1F        *hFADC[kNmods];	//!
	TH2F        *hTPK[kNmods];	//!
	TH2F        *hTQ[kNmods];	//!
	TH2F        *hTOFTQ[kNmods];	//!
#endif
	ClassDef(UserAnalysis, 1)   // A class to analyze event
};

#endif

